import bincopy
import struct

# goal is to extend the functionality and update the default creation parameter of a BinFile object.

class MemFile(bincopy.BinFile):
    def __init__(self, filenames=None, overwrite=True, word_size_bits=16, header_encoding='utf-8'):
        super().__init__(filenames=filenames, overwrite=overwrite, word_size_bits=word_size_bits, header_encoding=header_encoding)


    @staticmethod
    def read_evb_bin_file(evb_bin_file, at_address=0x2400):
        # EVB BIN file format is little-endian, while bin_copy; the hex file package expects big-endian!
        # so here we do the trick with unpack/pack!
        with open(evb_bin_file, mode='rb') as file: # b is important -> binary
            fileContent = file.read()

        fmt_in = "<{:d}H".format (int(len(fileContent)/2))
        fmt_out = ">{:d}H".format (int(len(fileContent)/2))
        data = list(struct.unpack(fmt_in, fileContent))
        bin_data = struct.pack(fmt_out, *data)

        bin_file = MemFile()
        bin_file.add_binary(bin_data, address=at_address, overwrite=True)
        return bin_file


    def add_32(self, address, int_data):
        if type(int_data) is list:
            byte_array = struct.pack('>{}L'.format(len(int_data)), *int_data)
        else:
            byte_array = struct.pack('>L', int_data)
        # silly mistake for 90632 in the eeprom map... LWORD is before HWORD
        for i in range(0, len(byte_array), 4):
            self.add_binary(byte_array[i+2:i+4], address=address, overwrite=True)
            self.add_binary(byte_array[i+0:i+2], address=address+1, overwrite=True)
        #
        # for new design use this:
        # self.add_binary(byte_array, address=address, overwrite=True)


    def add_32s(self, address, int_data):
        if type(int_data) is list:
            byte_array = struct.pack('>{}l'.format(len(int_data)), *int_data)
        else:
            byte_array = struct.pack('>l', int_data)
        # silly mistake for 90632 in the eeprom map... LWORD is before HWORD
        for i in range(0, len(byte_array), 4):
            self.add_binary(byte_array[i+2:i+4], address=address, overwrite=True)
            self.add_binary(byte_array[i+0:i+2], address=address+1, overwrite=True)
        #
        # for new design use this:
        # self.add_binary(byte_array, address=address, overwrite=True)


    def add_16(self, address, int_data):
        if type(int_data) is list:
            byte_array = struct.pack('>{}H'.format(len(int_data)), *int_data)
        else:
            byte_array = struct.pack('>H', int_data)
        self.add_binary(byte_array, address=address, overwrite=True)


    def add_16s(self, address, int_data):
        if type(int_data) is list:
            byte_array = struct.pack('>{}h'.format(len(int_data)), *int_data)
        else:
            byte_array = struct.pack('>h', int_data)
        self.add_binary(byte_array, address=address, overwrite=True)


    def write_evb_bin_file(self, evb_bin_file_name = "evb90632.bin"):
        if evb_bin_file_name is None:
            return None

        if not evb_bin_file_name.endswith('.bin'):
            evb_bin_file_name += '.bin'

        bin_data_in = self.as_binary()
        fmt_in = ">{:d}H".format (int(len(bin_data_in)/2))
        fmt_out = "<{:d}H".format (int(len(bin_data_in)/2))

        data = list(struct.unpack(fmt_in, bin_data_in))
        bin_data = struct.pack(fmt_out, *data)

        with open(evb_bin_file_name, "wb") as out_file:
            out_file.write(bin_data)

        return evb_bin_file_name


    def write_hex_file(self, hex_file_name = "evb90632.hex"):
        if hex_file_name is None:
            return self.as_ihex()

        if not hex_file_name.endswith('.hex'):
            hex_file_name += '.hex'

        with open(hex_file_name, "w") as out_file:
            out_file.write(self.as_ihex())

        return hex_file_name


    def get_address_data_pairs(self):
        pairs = []
        for seg in self.segments:
            for addr in range(int((seg.maximum_address-seg.minimum_address) / seg._word_size_bytes)):
              addr += int(seg.minimum_address / seg._word_size_bytes)
              pairs.append ((addr, self[addr]))
        return pairs




from mlx90632.mlx90632 import Mlx90632
from datetime import datetime
import sys


def main():
  if len(sys.argv) <= 1:
    print("Usage:")
    print("mlx90632-dump <interface> <read_count>\n")

    print("The first argument needs to be the url of the interface to connect (e.g. I2C-1, ftdi://ftdi:2232/1, mlx://evb:90632/1)")
    print("  This argument can also be 'auto', works only when 1 comport on system\n")
    print("examples:")
    print(" - mlx90632-dump mlx://evb:90632/2        ==> read 10 frames from 2nd EVB90632 connected to the computer")
    print(" - mlx90632-dump I2C-1                    ==> read 10 frames on I2C-1 bus on single board computer")
    print(" - mlx90632-dump ftdi://ftdi:2232/1       ==> read 10 frames on FTDI FT232H or FT2232H on USB port")
    print(" - mlx90632-dump auto                     ==> read 10 frames from 1st EVB90632")
    print(" - mlx90632-dump auto 1                   ==> read 1 frame from 1st EVB90632")
    print(" - mlx90632-dump 100                      ==> read 100 frames")
    print(" - mlx90632-dump 1000                     ==> read 1000 frames")
    print(" - mlx90632-dump 100                      ==> read 100 frames")
    print(" - mlx90632-dump 0                        ==> read 0 frames -> find out which comport is assigned")
    print("")
    exit(1)

  max_readings = 10
  interface = 'auto'

  if len(sys.argv) >= 2:
    if sys.argv[1].isdigit():
      max_readings = int(float(sys.argv[1]))
    else:
      interface = sys.argv[1]

  if len(sys.argv) >= 3:
    if sys.argv[2].isdigit():
      max_readings = int(float(sys.argv[2]))

  dev = Mlx90632(interface)
  dev.init()
  dev.read_chipid()
  print ("chipid = {}".format (dev.chipid))
  print ("chipid = {}".format (dev.chipid_str))

  reading_count = 0
  previous_time = datetime.now()
  dev.reset()

  print ("\n\nReading {}x".format(max_readings))


  while reading_count < max_readings:
    raw_data = None
    try:
      if dev.wait_new_data(2):
        raw_data = dev.read_measurement_data()
        dev.reset()
        dev.set_brownout()

      # print (raw_data)
    except Exception as e:
      dev.clear_error()
      print(e)
      pass

    if raw_data is not None:
      ta, to = dev.do_compensation(raw_data)
      now_time = datetime.now()
      delta_time = now_time - previous_time
      previous_time = now_time

      print("TA = {:6.2f}  | TO = {:6.2f}  | VddMon = {:6.2f}  -- {}".format (ta, to, dev.read_vddmonitor(), str(delta_time)))
  
      reading_count += 1

  dev.disconnect()



if __name__ == '__main__':
    main()



from pyftdi.i2c import I2cController
import time
import struct
import os
from mlx90632.hw_i2c_hal import HwI2cHalMlx90632
import platform


class HwFtdi2232h(HwI2cHalMlx90632):
    support_buffer = False
    pass

    def __init__(self, channel=1):
        try:
            ch_nr = int(channel)
            channel = 'ftdi://ftdi:2232h/{}'.format(ch_nr)
        except:
            pass
        self.i2c = I2cController()
        self.i2c.configure(channel)

    def connect(self):
        pass

    def disconnect(self):
        pass

    def i2c_read(self, i2c_addr, addr, count=1, unpack_format='H'):

        slave = self.i2c.get_port(i2c_addr)

        addr_msb = addr >> 8 & 0x00FF
        addr_lsb = addr & 0x00FF

        read = slave.exchange(struct.pack(">1H", addr), 2*count)

        if unpack_format is None:
            return bytes(list(read))
        results = struct.unpack(">{}{}".format(count, unpack_format), bytes(list(read)))
        if count == 1:
            return results[0]
        return results

    def i2c_write(self, i2c_addr, addr, data):
        data_bytes = bytearray()
        slave = self.i2c.get_port(i2c_addr)

        if type(data) is list:
          for d in data:
            data_bytes.append((d>>8) & 0x00FF)
            data_bytes.append(d & 0x00FF)
        else:
            data_bytes.append((data>>8) & 0x00FF)
            data_bytes.append(data & 0x00FF)

        slave.write(struct.pack(">1H", addr) + data_bytes)

        if (addr & 0xFF00) == 0x2400: # Wait after EEWRITE!
            time.sleep(0.010) # 10ms

        return 0

    def get_hardware_id(self):
        return "FT2232H I2C master"


def main():
    i2c = HwFtdi2232h()
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)



    i2c.i2c_write(0x3A, 0x3004, 0x0006)
    i2c.i2c_write(0x3A, 0x3005, 0x0006)


    d = i2c.i2c_read (0x3A, 0x3004, 2) # example to read 2 first words...
    print (d)

    i2c.i2c_write(0x3A, 0x3004, 0x0006)
    d = i2c.i2c_read (0x3A, 0x3004, 2) # example to read 2 first words...
    print (d)


    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)
    d = i2c.i2c_read (0x3A, 0x4000, 9, 'h') # example to read 9 first words...
    print (d)




if __name__ == '__main__':
    main()



"""A python interface for Mlx90632."""

from datetime import datetime
from sys import platform
import atexit

# MLX90632 mode
STEP_SLEEP = 1
STEP = 2
CONTINIOUS = 3

# ACCURACY_RANGE
MEDICAL = 1
COMMERCIAL = 2


class Mlx90632:
    def __init__(self, hw, i2c_addr=0x3A):
        self.word_size_bits = 16
        self.startup = False
        self.use_only_latest_IR_adc = False
        self.reload_calibration_data_on_brownout = True
        self.wait_new_data_until_end_of_full_cycle = False
        self.hw = None
        if hw is None or hw == 'auto':
            hw = "mlx://evb:90632/1"
        if isinstance(hw, str):
            if hw.startswith("I2C-"):
                from mlx90632.hw_rpi_gpio_i2chw import HwRpiI2cHw
                # from hw_rpi_gpio_i2chw import HwRpiI2cHw
                self.hw = HwRpiI2cHw(hw)
            if hw.startswith("I2CBB-"):
                from mlx90632.hw_rpi_gpio_bitbang import HwRpiGpioBitBang
                # from hw_rpi_gpio_bitbang import HwRpiGpioBitBang
                self.hw = HwRpiGpioBitBang(hw)
            if hw.startswith("ftdi://ftdi:2232"):
                from mlx90632.hw_ftdi_2232h import HwFtdi2232h
                # from hw_ftdi_2232h import HwFtdi2232h
                self.hw = HwFtdi2232h(hw)
            if hw.startswith("mlx://evb:90632"):
                from mlx90632.hw_usb_evb90632 import HwUsbEvb90632
                # from hw_usb_evb90632 import HwUsbEvb90632
                self.hw = HwUsbEvb90632(hw)

        else:
            self.hw = hw
        self.i2c_addr = i2c_addr

        self.connect()

        atexit.register(self.disconnect)


    def init(self):
        self.read_status()
        self.clear_new_data()
        self.read_status()
        self.read_control()
        self.read_calibration_data()

    def read_calibration_data(self):
        
        trim_version = 0
        dsp_version = 0
        cal_Ea = 74.0
        cal_Eb = 22260.0
        cal_Fa = 7e-7
        cal_Ga = -0.0005
        cal_Fb = -0.0004
        cal_Gb = 0
        cal_Ka = 0

        w = self.hw.i2c_read (self.i2c_addr, 0x240B)
        trim_version = w >> 8
        dsp_version = w & 0x0FF

        w = self.hw.i2c_read (self.i2c_addr, 0x2424)
        EE_cal_Ea = w
        w = self.hw.i2c_read (self.i2c_addr, 0x2425)
        EE_cal_Ea += (w * 2**16)
        cal_Ea = EE_cal_Ea * 2**-16

        w = self.hw.i2c_read (self.i2c_addr, 0x2426)
        EE_cal_Eb = w
        w = self.hw.i2c_read (self.i2c_addr, 0x2427)
        EE_cal_Eb += (w * 2**16)
        cal_Eb = EE_cal_Eb * 2**-8
      
        w = self.hw.i2c_read (self.i2c_addr, 0x2428)
        EE_cal_Fa = w
        w = self.hw.i2c_read (self.i2c_addr, 0x2429)
        EE_cal_Fa += (w * 2**16)
        cal_Fa = EE_cal_Fa * 2**-46
      
        w = self.hw.i2c_read (self.i2c_addr, 0x242A)
        EE_cal_Fb = w
        w = self.hw.i2c_read (self.i2c_addr, 0x242B)
        EE_cal_Fb += (w * 2**16)
        if (EE_cal_Fb & 0x80000000):
          EE_cal_Fb -= 2**32
        cal_Fb = EE_cal_Fb * 2**-36
      
        w = self.hw.i2c_read (self.i2c_addr, 0x242C)
        EE_cal_Ga = w
        w = self.hw.i2c_read (self.i2c_addr, 0x242D)
        EE_cal_Ga += (w * 2**16)
        if (EE_cal_Ga & 0x80000000):
          EE_cal_Ga -= 2**32
        cal_Ga = EE_cal_Ga * 2**-36
      
        w = self.hw.i2c_read (self.i2c_addr, 0x242E)
        EE_cal_Gb = w
        if (EE_cal_Gb & 0x8000):
          EE_cal_Gb -= 2**16
        cal_Gb = EE_cal_Gb * 2**-10
      
        w = self.hw.i2c_read (self.i2c_addr, 0x242F)
        EE_cal_Ka = w
        if (EE_cal_Ka & 0x8000):
          EE_cal_Ka -= 2**16
        cal_Ka = EE_cal_Ka * 2**-10
      
        w = self.hw.i2c_read (self.i2c_addr, 0x2481)
        EE_cal_Ha = w
        if (EE_cal_Ha & 0x8000):
          EE_cal_Ha -= 2**16
        cal_Ha = EE_cal_Ha * 2**-14
      
        w = self.hw.i2c_read (self.i2c_addr, 0x2482)
        EE_cal_Hb = w
        if (EE_cal_Hb & 0x8000):
          EE_cal_Hb -= 2**16
        cal_Hb = EE_cal_Hb * 2**-10

        w = self.hw.i2c_read (self.i2c_addr, 0x2430)
        EE_cal_VddMonOffset = w
        if (EE_cal_VddMonOffset & 0x8000):
          EE_cal_VddMonOffset -= 2**16
        cal_VddMonOffset = EE_cal_VddMonOffset

            
        self.calib_data = {
                'cal_Ea'  : cal_Ea, 
                'cal_Eb'  : cal_Eb, 
                'cal_Fa'  : cal_Fa, 
                'cal_Ga'  : cal_Ga, 
                'cal_Fb'  : cal_Fb, 
                'cal_Gb'  : cal_Gb, 
                'cal_Ka'  : cal_Ka, 
                'cal_Ha'  : cal_Ha,
                'cal_Hb'  : cal_Hb,
                'cal_VddMonOffset'  : cal_VddMonOffset,
                'EE_cal_Ea'  : EE_cal_Ea, 
                'EE_cal_Eb'  : EE_cal_Eb, 
                'EE_cal_Fa'  : EE_cal_Fa, 
                'EE_cal_Ga'  : EE_cal_Ga, 
                'EE_cal_Fb'  : EE_cal_Fb,
                'EE_cal_Gb'  : EE_cal_Gb,
                'EE_cal_Ka'  : EE_cal_Ka,
                'EE_cal_Ha'  : EE_cal_Ha,
                'EE_cal_Hb'  : EE_cal_Hb,
                'EE_cal_VddMonOffset'  : EE_cal_VddMonOffset,
                'dsp_version'   : dsp_version,
                'trim_version'  : trim_version
        }
        self.set_brownout(use_cache=False)
        return self.calib_data

    def set_vdd(self, vdd):
        """Set Vdd of the sensor"""
        # if supported...
        if callable(getattr(self.hw, 'set_vdd', None)):
            self.hw.set_vdd(vdd)

    def get_refresh_rate(self):
        return self.frame_rate

    def clear_error(self):
        if callable(getattr(self.hw, 'clear_error', None)):
            self.hw.clear_error(self.i2c_addr)

    def write_ee_refresh_rate(self, refresh_rate, unit='Hz'):
        """
        Write the refresh rate to EEPROM
        :param frame_rate: the new frame rate for the sensor
        :param unit: 'Hz' or 'code'.
        """

        refresh_rate_code = refresh_rate
        if unit == 'Hz':
            refresh_rate_code = 2
            LUT = {64   : 7,
                   32   : 6, 
                   16   : 5, 
                    8   : 4, 
                    4   : 3, 
                    2   : 2, 
                    1   : 1, 
                    0.5 : 0, 
                  }
            if refresh_rate in LUT:
                refresh_rate_code = LUT[refresh_rate]

        ee_meas1 = self.hw.i2c_read (self.i2c_addr, 0x24E1)
        ee_meas2 = self.hw.i2c_read (self.i2c_addr, 0x24E2)

        ee_meas1 &= ~(0x07 << 8)
        ee_meas2 &= ~(0x07 << 8)

        ee_meas1 |= (refresh_rate_code & 0x07) << 8
        ee_meas2 |= (refresh_rate_code & 0x07) << 8

        self.write_ee(0x24E1, ee_meas1)
        self.write_ee(0x24E2, ee_meas2)


    def write_ee(self, address, data):
        """
          Writing to EEPROM is prohibited during application run; use only at end-of-line calibration.

        """
        old_data = self.hw.i2c_read (self.i2c_addr, address)
        if data != old_data: # if update required
            # erase the address when not yet zero
            if old_data != 0:
                self.hw.i2c_write (self.i2c_addr, 0x3005, 0x554C)
                self.hw.i2c_write (self.i2c_addr, address, 0x0000)
                self.hw.i2c_write (self.i2c_addr, 0x3005, 0x0006)

            # write the new data (when new data is different from zero)
            if data != 0:
                self.hw.i2c_write (self.i2c_addr, 0x3005, 0x554C)
                self.hw.i2c_write (self.i2c_addr, address, data)
                self.hw.i2c_write (self.i2c_addr, 0x3005, 0x0006)


    def measure_vdd(self):
        """
        Measure Vdd of the sensor

        @:return vdd
        """
        if callable(getattr(self.hw, 'measure_vdd', None)):
            return self.hw.measure_vdd()
        return None


    def get_hardware_id(self):
        """
        :return: bytes array representing the hardware id
        """
        return self.hw.get_hardware_id()


    def connect(self):
        """
        Do the necessary initialisation before measurements are taken

        procedure:
            poll until HW id received (timeout??)
            init SW I2C
            set vdd
            I2C:
                set refresh rate
                start conversion
            set evb refresh rate
        """
        self.hw.connect()
        self.reset()


    def disconnect(self):
        """
        """
        self.hw.disconnect()


    def i2c_read(self, addr, count=1, unpack_format='H'):
        results = self.hw.i2c_read(self.i2c_addr, addr, count, unpack_format)
        if count == 1:
            return results
        return list(results)


    def reset(self):
        # Soft reset via I2C command...
        self.hw.i2c_write (self.i2c_addr, 0x3005, 0x0006)
        self.startup = True


    def read_vddmonitor_data(self):
        raw_data = self.hw.i2c_read (self.i2c_addr, 0x4000, 3, 'h')
        # print (raw_data)
        return raw_data


    def read_measurement_data(self):
        raw_data = self.hw.i2c_read (self.i2c_addr, 0x4003, 6, 'h')
        self.clear_new_data(False)
        # print (raw_data)
        return raw_data


    def read_status(self):
        self.reg_status = self.hw.i2c_read (self.i2c_addr, 0x3FFF)
        self.reg_status_brownout = self.reg_status & 0x0100 != 0
        self.reg_status_cycle_position = (self.reg_status & 0x007C) >> 2
        self.reg_status_new_data = self.reg_status & 0x0001 != 0
        return self.reg_status


    def read_control(self):
        self.reg_control = self.hw.i2c_read (self.i2c_addr, 0x3001)
        self.reg_control_mode = (self.reg_status & 0x006) >> 1
        self.reg_control_soc = (self.reg_status & 0x008) >> 3
        self.reg_control_table_select = (self.reg_status & 0x01F0) >> 4
        return self.reg_control


    def read_ee_product_code(self):
        self.ee_product_code = self.hw.i2c_read (self.i2c_addr, 0x2409)
        self.ee_product_code_accuracy_range = (self.ee_product_code & 0x007) >> 0
        return self.ee_product_code


    def read_ee_pc_accuracy_range(self):
        self.read_ee_product_code()
        return self.ee_product_code_accuracy_range


    def wait_new_data(self, timeout_seconds=-1):
        timeout = False
        start_date_time = datetime.now()
        while not timeout:
          if timeout_seconds >= 0:
            delta_time = (datetime.now() - start_date_time).total_seconds()
            if delta_time > timeout_seconds:
                timeout = True # but give the new_data bit one more chance!
          self.read_status()
          if self.reg_status_new_data:
            if self.startup or self.wait_new_data_until_end_of_full_cycle:
              if self.reg_status_cycle_position != 2:
                continue # wait longer, untill we have a full measurement cycle; at first time after startup; or when we require a full cycle!
            self.startup = False
            return True
        return False


    def poll_new_data(self):
        self.read_status()
        if self.reg_status_new_data:
          return True
        return False


    def clear_new_data(self, use_cache=True):
        if not use_cache:
            self.read_status()
        new_status_value = self.reg_status & ~0x0001
        self.hw.i2c_write (self.i2c_addr, 0x3FFF, new_status_value)


    def set_brownout(self, use_cache=True):
        if not use_cache:
            self.read_status()
        new_status_value = self.reg_status | 0x0100
        self.hw.i2c_write (self.i2c_addr, 0x3FFF, new_status_value)


    def write_control_mode(self, mode=CONTINIOUS):
        self.read_control()
        new_control_value = self.reg_control & ~0x0006
        new_control_value |= ((mode & 0x03) << 1) 
        self.hw.i2c_write (self.i2c_addr, 0x3001, new_control_value)


    def write_control_soc(self, value=True):
        self.read_control()
        new_control_value = self.reg_control & ~0x0008
        if value:
            new_control_value |= 0x0008
        self.hw.i2c_write (self.i2c_addr, 0x3001, new_control_value)


    def do_compensation(self, raw_data):
        """
        Calculates the temperatures based on raw_data and calibration_data
        :param raw_data: the raw data from 'read_measurement_data'
        :return: the calculated TA/TO
        """
        if self.reload_calibration_data_on_brownout:
            if self.reg_status_brownout:
                self.read_calibration_data()

        Ea = self.calib_data['cal_Ea']   
        Eb = self.calib_data['cal_Eb']  
        Fa = self.calib_data['cal_Fa']    
        Ga = self.calib_data['cal_Ga']   
        Gb = self.calib_data['cal_Gb']   
        Fb = self.calib_data['cal_Fb']   
        Ha = self.calib_data['cal_Ha']   
        Hb = self.calib_data['cal_Hb']   
        Ka = self.calib_data['cal_Ka']   
    
        Xa = raw_data[5] + Gb * raw_data[2] / 4 / 3.0
        Xb = raw_data[5] + Ka * raw_data[2] / 4 / 3.0

        Ya = raw_data[2] / 4 / 3.0 / Xa * 2**19

        Xc = None
        if self.use_only_latest_IR_adc:
            if self.reg_status_cycle_position == 1:
                Xc = (raw_data[0] + raw_data[1])/2
            if self.reg_status_cycle_position == 2:
                Xc = (raw_data[3] + raw_data[4])/2
        else:
            Xc = (raw_data[0] + raw_data[1] + raw_data[3] + raw_data[4])/4

        Yb = Xc / 4 / 3.0 / Xb * 2**19
    
        if Ea == 0:
          return (-999,-999)
    
        TA_computed = (Ya - Eb) / Ea + 25
        TO_computed = 34
    
        # iterate 3 times:
        for i in range (3):
          Za = Ha * Fa * (1 + Ga * (TO_computed - 25) + Fb * (TA_computed - 25))
          TO_computed = (Yb/Za + (TA_computed+273.15)**4)**0.25 - 273.15 - Hb
    
        return (TA_computed, TO_computed)


    def read_temperature(self):
        raw_data = self.read_measurement_data()
        return self.do_compensation(raw_data)


    def read_chipid(self):
        self.chipid_words = self.hw.i2c_read (self.i2c_addr, 0x2405, 4)
        self.chipid = self.chipid_words[0] + self.chipid_words[1] * (2**16) + self.chipid_words[2] * (2**32)
        self.chipid_str = "{:04X}-{:04X}-{:04X}".format(self.chipid_words[2], self.chipid_words[1], self.chipid_words[0])
        return self.chipid


    def read_vddmonitor(self):
        raw_data = self.read_vddmonitor_data()
        if self.reload_calibration_data_on_brownout:
            if self.reg_status_brownout:
                self.read_calibration_data()
        cal_VddMonOffset = self.calib_data['cal_VddMonOffset']
        if cal_VddMonOffset < 10000:
            cal_VddMonOffset = 22841
        if cal_VddMonOffset == 22841:
            cal_VddMonOffset -= 150
        vddmon = (raw_data[0] + raw_data[1])/2
        vddmon -= cal_VddMonOffset
        vddmon /= 6920
        vddmon += 3.3
        return vddmon


    def force_read_vddmonitor(self):
        self.reset() # a reset will 'force' to run the initial steps in the table!
        self.clear_new_data()
        while True:
            self.read_status()
            if self.reg_status_new_data:
                break
        return self.read_vddmonitor()


def main():
    # demo to test the file.. get a single reading from EVB90632.
    # Should you want a more user friendly example, please have a look at Mlx90632_dump.py example.
    import time
    from MemFile import MemFile

        # evb_bin_file_name = evb_bin_file_name.format(id=self.chipid_str)

    # defaults
    max_readings = 10

    # dev = Mlx90632("I2C-1")
    # dev = Mlx90632("ftdi://ftdi:2232/1")
    dev = Mlx90632("mlx://evb:90632/1")

    dev.init()
    dev.read_chipid()
    # dev.write_ee_refresh_rate(2)

    print ("chipid = {}".format (dev.chipid))
    print ("chipid = {}".format (dev.chipid_str))

    # example to read entire EEPROM and write to file (hex- & bin-file)
    print ("\n\nreading EEPROM content to {}.hex/.bin".format(dev.chipid_str))
    m = MemFile()
    word_data = dev.i2c_read(0x2400, 256, 'H')
    m.add_16(0x2400, word_data)
    m.write_evb_bin_file(dev.chipid_str)
    m.write_hex_file(dev.chipid_str)
    print ("done.")


    # example read memory file and compare with eeprom content read
    print ('\n\nreading eeprom memory dump, and compare with actual content...')
    h = MemFile(dev.chipid_str + '.hex')
    # h = MemFile(dev.chipid_str + '.bin')
    print ("")
    print ("addr: data")
    print ("----  ----")
    word_data = dev.i2c_read(0x2400, 256, 'H')
    changed = False
    for p in h.get_address_data_pairs():
        data = word_data.pop(0)
        print ("{:04X}: {:04X} -- {:04X} -- {}".format (p[0], p[1], data, data==p[1]))
        if data != p[1]:
            changed = True
    if changed:
        print ("data differs!")
    else:
        print ("data matches!")

    print ('accuracy_range: {}'.format(dev.read_ee_pc_accuracy_range()))

    reading_count = 0
    previous_time = datetime.now()
    dev.reset()

    print ("\n\nReading {}x".format(max_readings))


    while reading_count < max_readings:
        raw_data = None
        try:
            if dev.wait_new_data(2):
                raw_data = dev.read_measurement_data()
                dev.reset()
                dev.set_brownout()

            # print (raw_data)
        except Exception as e:
            dev.clear_error()
            print(e)
            pass

        if raw_data is not None:
            ta, to = dev.do_compensation(raw_data)
            now_time = datetime.now()
            delta_time = now_time - previous_time
            previous_time = now_time

            print("TA = {:6.2f}  | TO = {:6.2f}  | VddMon = {:6.2f}  -- {}".format (ta, to, dev.read_vddmonitor(), str(delta_time)))
        
            reading_count += 1

    dev.disconnect()


if __name__ == '__main__':
    main()



import struct

class HwI2cHalMlx90632:
    support_buffer = False
    sensor_type = None
    pass

    def connect(self):
        print("connect function not implemented")

    def disconnect(self):
        print("disconnect function not implemented")

    def i2c_read(self, i2c_addr, addr, count=1):
        print("i2c_read function not implemented")
        return bytes([0] * count), 0

    def i2c_write(self, i2c_addr, addr, data):
        print("i2c_write function not implemented")
        return 0

    def get_sensor_type(self, i2c_addr):
        sensor, stat = self.i2c_read(i2c_addr, 0x240A, 2)
        sensor = struct.unpack(">H",sensor)[0]
        self.sensor_type = (sensor & 0x40) >> 6
        return self.sensor_type

    def get_hardware_id(self):
        return "HardWare Abstraction Layer (dummy)"



from mlx90632.pympt.core import *
from mlx90632.pympt.channel import MptChannel

import sys

__all__ = ["MptException", "BadCrcException", "CommandTooLongException",
           "NotConnectedException", "I2CAcknowledgeError", "MptChannel"]

if sys.platform == "linux":
    from mlx90632.pympt.UsbHidChannel import UsbHidChannel
    __all__.append("UsbHidChannel")
else:
    from mlx90632.pympt.PyWinUsbChannel import PyWinUsbChannel
    __all__.append("PyWinUsbChannel")



"""MptChannel for USB-HID using pywinusb class device"""

from mlx90632.pympt.core import *
from mlx90632.pympt.channel import *

import pywinusb.hid as hid

import queue
import time


DEBUG_PRINT=False


class PyWinUsbChannel(MptChannel):
    def __init__(self):
        self.in_buffer = b""
        self.dev = None
        self.in_report = None
        self.out_report = None
        self.hid_device_index = 0
        self.in_queue = queue.Queue()


    def in_handler(self, data):
        if DEBUG_PRINT:
            print ('in_handler', len(data), list(data[:10]), '...')
        self.in_queue.put(data[1:])


    def connect(self, hid_device_index):
        """Initialize USB connection to EVB90632"""
        if hid_device_index is None:
            hid_device_index = self.hid_device_index

        # find EVB90632
        all_devices = hid.HidDeviceFilter(vendor_name = "Melexis", product_name="EVB90632").get_devices()

        if len(all_devices) <= hid_device_index:
            return len(all_devices)
        
        self.dev = all_devices[hid_device_index]

        self.dev.open()

        self.in_report = self.dev.find_input_reports()[0]
        self.out_report = self.dev.find_output_reports()[0]
        self.dev.set_raw_data_handler(self.in_handler)


    def disconnect(self):
        if self.dev is not None:
            self.dev.close()
        self.dev = None
        self.in_report = None
        self.out_report = None
 

    def flush_buffers(self):
        self.in_buffer = b""
        self.in_queue.queue.clear()


    def write(self, data):
        if self.out_report is None:
            raise NotConnectedException()
        # HID reports must be sent with full buffer size data -> 64 bytes

        if not self.dev.is_plugged():
            raise NotConnectedException()

        n = len(data)
        if n % 64 != 0:
            data += b"\x00" * (64 - n % 64)
        for i in range(0, len(data), 64):
            if DEBUG_PRINT:
                print('Sending: {} ...'.format(list(data[i:i+10])))
            self.out_report.set_raw_data(bytes([0]) + data[i:i+64])
            self.out_report.send()
        if DEBUG_PRINT:
            print('----------')


    def read(self, nbytes):
        if self.in_report is None:
            raise NotConnectedException()

        if not self.dev.is_plugged():
            raise NotConnectedException()

        # we have setup the in_hander function, each time there is new data a new item appears in the in_queue
        # so all we have to do now is to get a message from that queue
        res_data = self.in_buffer
        self.in_buffer = b""
        while len(res_data) < nbytes:
            data = self.in_queue.get(block=True, timeout=5)
            if DEBUG_PRINT:
                print('PyWinUsbChannel: read:', data[:10], '...')
            res_data += bytearray(data)
        n = len(res_data) - nbytes
        if n > 0:
            self.in_buffer = res_data[-n:]
            res_data = res_data[:-n]
        if DEBUG_PRINT:
            print('PyWinUsbChannel: res_data:', list(res_data[:10]), '...')
        return res_data



import time
import struct
import os
import sys
from mlx90632.hw_i2c_hal import HwI2cHalMlx90632
import platform

import mlx90632.pympt


DEFAULT_I2C_TCLOCK    =    10.0
DEFAULT_I2C_TSTART    =    10.0
DEFAULT_I2C_TSTOP     =    10.0
DEFAULT_I2C_TWRDELAY  =    10.0
DEFAULT_I2C_THOLDDATA =     0.05


class Evb90632Commands:
    CMD_ResetHardware = bytes([0])
    CMD_GetHardwareID = bytes([1])
    CMD_GetSoftwareID = bytes([3])
    CMD_I2C_Config = bytes([30])
    CMD_I2C_Master_SW = bytes([31])
    CMD_Hello = bytes([35])
    CMD_SetVdd = bytes([150])
    CMD_MeasureVddIdd = bytes([156])


class HwUsbEvb90632(HwI2cHalMlx90632):
    def __init__(self, channel=0):
        ch_index = channel
        try:
            ch_index = int(channel[len("mlx://evb:90632/"):]) - 1
        except:
            ch_index = channel
            pass
        if sys.platform == "win32":
            self.channel = mlx90632.pympt.PyWinUsbChannel()
        else:
            self.channel = mlx90632.pympt.UsbHidChannel()
        self.channel.connect(ch_index)
        self.set_i2c_config()
        self.set_vdd (0.0)
        time.sleep(0.01)
        self.set_vdd (3.3)
        time.sleep(0.01)


    def set_i2c_config(self, clock=DEFAULT_I2C_TCLOCK, start=DEFAULT_I2C_TSTART, stop=DEFAULT_I2C_TSTOP, wr_delay=DEFAULT_I2C_TSTOP, t_hold_data=DEFAULT_I2C_THOLDDATA):
        MCK = 48054857 # Hz
        l_clock = int(round(clock / 1000000 / 12 * MCK - 39 / 12 + 1))
        if l_clock < 1:
            l_clock = 1
        l_start = int(round(start / 1000000 / 6 * MCK - 19 / 6 + 1))
        if l_start < 0:
            l_start = 0
        l_stop =int(round(stop / 1000000 / 6 * MCK - 10 / 6 + 1))
        if l_stop < 0:
            l_stop = 0
        l_wr_delay = int(round(wr_delay / 1000000 / 6 * MCK - 26 / 6 + 1))
        if l_wr_delay < 0:
            l_wr_delay = 0
        l_t_hold_data = int(round(t_hold_data / 1000000 / 6 * MCK - 10 / 6 + 1))
        if l_t_hold_data < 0:
            l_t_hold_data = 0

        cmd = Evb90632Commands.CMD_I2C_Config
        cmd += struct.pack("<lllll", l_clock, l_start, l_stop, l_wr_delay, l_t_hold_data)

        result = self.channel.send_command(cmd)
        if result is None:
            return False
        if len(result) == 1:
            if result[0] == cmd[0]:
                return None
        return False


    def get_hardware_id(self):
        hardware_id = self.channel.send_command(Evb90632Commands.CMD_GetHardwareID)
        r = {}
        r['vid'] = struct.unpack('<H', hardware_id[1:3])[0]
        r['pid'] = struct.unpack('<H', hardware_id[3:5])[0]

        (r['vendor_name'], r['product_name'], r['version'], d) = hardware_id[6:].decode().split('\x01', 3)

        return r


    def get_software_id(self):
        software_id = self.channel.send_command(Evb90632Commands.CMD_GetSoftwareID)

        r = {}
        (r['template_name'], r['template_version'], r['template_timestamp'], r['firmware_name'], r['firmware_version'], r['firmware_timestamp'], d) = software_id[1:].decode().split('\x01')
        return r


    def set_vdd(self, vdd=3.3):
        """Set Vdd of the sensor"""
        cmd = Evb90632Commands.CMD_SetVdd + bytes([0])
        cmd = cmd + struct.pack("<f", float(vdd))
        self.channel.send_command(cmd)


    def measure_vdd(self):
        """
        Measure Vdd&Idd of the sensor

        @:return (vdd, idd)
        """
        data = self.channel.send_command(Evb90632Commands.CMD_MeasureVddIdd)
        vdd = struct.unpack('<f', data[1:5])[0]
        idd = struct.unpack('<f', data[5:9])[0]
        return (vdd, idd)


    def get_hello(self):
        hello = self.channel.send_command(Evb90632Commands.CMD_Hello)
        return hello[1:].decode()


    def connect(self):
        pass


    def disconnect(self):
        return self.channel.disconnect()


    def i2c_read(self, i2c_addr, addr, count=1, unpack_format='H'):
        data_bytes = bytearray()

        cmd = Evb90632Commands.CMD_I2C_Master_SW
        cmd += struct.pack(">B", i2c_addr)
        cmd += struct.pack(">H", addr)
        cmd += struct.pack("<H", int(count*2))

        read = self.channel.send_command(cmd)

        if unpack_format is None:
            return bytes(list(read[1:]))

        results = struct.unpack(">{}{}".format(count, unpack_format), bytes(list(read[2:2+int(count*2)])))
        if count == 1:
            return results[0]
        return results

    def i2c_write(self, i2c_addr, addr, data):
        data_bytes = bytearray()

        if type(data) is list:
          for d in data:
            data_bytes.append((d>>8) & 0x00FF)
            data_bytes.append(d & 0x00FF)
        else:
            data_bytes.append((data>>8) & 0x00FF)
            data_bytes.append(data & 0x00FF)

        cmd = Evb90632Commands.CMD_I2C_Master_SW
        cmd += struct.pack(">B", i2c_addr)
        cmd += struct.pack(">H", addr)
        cmd += data_bytes
        cmd += struct.pack(">H", 0)

        result = self.channel.send_command(cmd)

        if result is not None and len(result) > 1:
            if result[0] == cmd[0]:
                return None
        return False


def main():
    i2c = HwUsbEvb90632()

    print (i2c.get_hello())

    print ("hardware_id:")
    for k, v in i2c.get_hardware_id().items():
        print ("- {:20s}: {}".format (k, v))

    print ("firmware_id:")
    for k, v in i2c.get_software_id().items():
        print ("- {:20s}: {}".format (k, v))

    print("READ1 ---------------------------------")
    d = i2c.i2c_read(0x3A, 0x4000, 1)
    print("0x4000: {} {:04X} {} {}".format(d, d, int(d/256), int(d%256)))
    d = i2c.i2c_read(0x3A, 0x4001, 1)
    print("0x4001: {} {:04X} {} {}".format(d, d, int(d/256), int(d%256)))
    print("READ2 ---------------------------------")
    print("0x4000-1: {}".format(i2c.i2c_read(0x3A, 0x4000, 2)))



if __name__ == '__main__':
    main()



"""MptChannel for USB-HID class device"""
from mlx90632.pympt.core import *
from mlx90632.pympt.channel import *
import time

try:
    import hid
# requires installing the binary package hidapi
# https://github.com/apmorton/pyhidapi#installing-hidapi
except ImportError as e:
    print ("The python package 'hid' requires installing the binary package hidapi")
    print ("https://github.com/apmorton/pyhidapi#installing-hidapi")
    print ("for debian/ubuntu: ")
    print ("")
    print ("sudo apt update")
    print ("sudo apt install libhidapi-libusb0")
    print ("")
    raise ImportError(e)


DEBUG_PRINT=False

VID = 0x03e9
PID = 0x0019


class UsbHidChannel(MptChannel):
    def __init__(self):
        self.in_buffer = b""
        self.hid = None
        self.timeout_ms = 300

    def connect(self, ch_index):
        """Initialize USB connection to EVB90632"""
        # find EVB90632

        hid_enumerate = hid.enumerate(VID, PID)
        if hid_enumerate is None:
            print ("no EVB found!")
            return False

        if len(hid_enumerate) <= ch_index:
            print ("no EVB at index {} found!".format(ch_index+1))
            return False

        self.hid = hid.Device(path=hid_enumerate[ch_index]['path'])

        if self.hid is None:
            print ("no EVB found!")
            return False

        self.flush_buffers()
        return True


    def disconnect(self):
        if self.hid is not None:
            self.flush_buffers()
            self.hid.close()
            self.hid = None


    def flush_buffers(self):
        self.in_buffer = b""
        while True:
            try:
                r = self.hid.read(64, 1)
                if r == b'':
                    break
            except Exception as e:
                break


    def write(self, data):
        self.flush_buffers()
        if self.hid is None:
            raise NotConnectedException()
        # HID reports must be sent with full buffer size data -> 64 bytes
        n = len(data)
        if n % 64 != 0:
            data += b"\x00" * (64 - n % 64)
        for i in range(0, len(data), 64):
            if DEBUG_PRINT:
                time.sleep(0.1)
                print('Sending: {}'.format(data[i:i+64]))
            self.hid.write(data[i:i+64])
        if DEBUG_PRINT:
            time.sleep(0.1)
            print('----------')


    def read(self, nbytes):
        if self.hid is None:
            raise NotConnectedException()
        res_data = self.in_buffer
        self.in_buffer = b""
        while len(res_data) < nbytes:
            try:
                data = self.hid.read(64, self.timeout_ms)
            except Exception as e:
                print ('error', e)
                continue

            if DEBUG_PRINT:
                time.sleep(0.1)
                print(data)
            res_data += data
        n = len(res_data) - nbytes
        if n > 0:
            self.in_buffer = res_data[-n:]
            res_data = res_data[:-n]
        return res_data
